#### Real (artificial) data application

#### main packages ####
library(rqPen)
library(quantreg)
library(qrcmNP)

#### Please, set your working directory first!

#### loading and managing data ####
X <- readRDS("artificial_data.rds")
dim(X)
summary(X)

#### figure 1 ####
oRQ <- rq(ic ~ age + hgt + bmi + sex + smoker + occ_exp + cough + wheeze + asthma, data=X, 
          tau=seq(.01,.99,.01))

olm <- lm(ic ~ age + hgt + bmi + sex + smoker + occ_exp + cough + wheeze + asthma, data=X)

temp <- summary(oRQ)
age <- sapply(1:99, function(i) temp[[i]]$coefficients[2, 1])
sdAge <- sapply(1:99, function(i) temp[[i]]$coefficients[2, 2])
alpha <- qnorm(1 - (1 - .95)/2)
lowAge <- age - alpha*sdAge
upAge <- age + alpha*sdAge

tempObj <- lm(age ~ seq(.01, .99, .01))
yy <- c(lowAge, tail(upAge, 1), rev(upAge), lowAge[1])
xx <- c(seq(.01, .99, .01), tail(seq(.01, .99, .01), 1), 
        rev(seq(.01, .99, .01)), seq(.01, .99, .01)[1])

# pdf("~/Desktop/age.pdf", width=.5*18, height=.5*18)
par(mfrow=c(1,1), mar=c(5,5,2,2)+.1)
plot(seq(.01, .99, .01), age, type="l", lwd=2.5, lty=1, cex.axis=1.3, cex.lab=1.5, xlab="", ylab="", 
     main="", ylim=range(yy))
mtext(expression(hat(beta)[Age](p)), side = 2, line = 2.5, cex = 1.5)
mtext("p", side = 1, line = 2.5, cex = 1.5)
polygon(xx, yy, col = gray(.8), border = NA)
lines(seq(.01, .99, .01), age, lwd=2.5, lty=1)
lines(seq(.01, .99, .01), fitted(tempObj), lty=3, lwd=3)
# dev.off()

#### table 1 ####
set.seed(1)
y <- X$ic
x <- as.matrix(X[, -10]); storage.mode(x) <- "double"
p <- c(.01,.05, .1, .2, .3, .4, .5, .6, .7, .8, .9, .95, .99)
beta <- matrix(NA, nrow=dim(x)[2]+1, ncol=length(p),
               dimnames=list(c("(Intercept)", "age", "hgt", "bmi", "sex", "smoker", "exposure", "cough",
                               "wheeze", "asthma"), p))
best <- vector(l=length(p))

xx <- scale(x)
for(i in 1:length(p)){
  o <- cv.rq.pen(xx, y, tau=p[i], criteria="BIC")
  best[i] <- which(o$cv$lambda %in% o$lambda.min)
  beta[, i] <- o$models[[best[i]]]$coef
}
round(beta, 8)

#### computing penalized qrcm estimator ####
s <- matrix(1, nrow=10, ncol=6); s[1, 4:6] <- 0; s[2:10, 2:3] <- 0
obj <- piqr(ic ~ ., data=X, formula.p = ~ I(log(p)) + I(log(1-p)) + slp(p, 3), s=s)

#### figure 4 ####
# pdf("IC.pdf", width=.5*20, height=.5*20)
par(mfrow=c(2,2), mar=c(5,5,4,2)+.1)
plot(obj, xvar="grad", lwd=2, cex.lab=1.8, cex.axis=1.5, col=gray.colors(72))
plot(obj, xvar="lambda", lwd=2, cex.lab=1.8, cex.axis=1.5, col=gray.colors(72))
bestObjAIC <- gof.piqr(obj, method="AIC", logi=T, plot=T, lwd=2.2, cex.lab=1.8, cex.axis=1.5)
bestObjBIC <- gof.piqr(obj, method="BIC", logi=T, plot=T, lwd=2.2, cex.lab=1.8, cex.axis=1.5)
par(mfrow=c(1,1))
# dev.off()

#### fitting unpenalized qrcm estimator ####
oIQRaic <- iqr(obj$internal$formula, data=obj$internal$data, formula.p=obj$internal$formula.p,
               s=1*I(bestObjAIC$betaMin != 0))
oIQRbic <- iqr(obj$internal$formula, data=obj$internal$data, formula.p=obj$internal$formula.p,
               s=1*I(bestObjBIC$betaMin != 0))

#### table 3 ####
set.seed(1234); tfAIC <- test.fit(oIQRaic, R=100); tfAIC
set.seed(1234); tfBIC <- test.fit(oIQRbic, R=100); tfBIC

aicPDF <- length(which(oIQRaic$PDF == 0))/length(oIQRaic$PDF)
bicPDF <- length(which(oIQRbic$PDF == 0))/length(oIQRbic$PDF)

tab3 <- matrix(NA, 2, 6, dimnames = list(NULL, c("Criterion", "lambda", "n. of parameters", 
                                                 "n. of covariates", "Loss", "P-value KS")))
tab3[,1] <- c("AIC", "BIC"); tab3[,2] <- round(c(bestObjAIC$minLambda, bestObjBIC$minLambda),2)
tab3[,3] <- c(paste0(sum(bestObjAIC$betaMin!=0), "/", sum(sum(s))), paste0(sum(bestObjBIC$betaMin!=0), "/", sum(sum(s))))
tab3[,4] <- c(paste0(bestObjAIC$dfMinLambda, "/", nrow(s)), paste0(bestObjBIC$dfMinLambda, "/", nrow(s)))
tab3[,5] <- round(c(oIQRaic$obj.function, oIQRbic$obj.function), 2)
tab3[,6] <- round(c(tfAIC[1,2], tfBIC[1,2]), 2)

#### table 4 ####
# tabAIC <- format(round(cbind(summary(oIQRaic)$coef, summary(oIQRaic)$se), 3), nsmall = 3, trim = T)
# tabAIC[tabAIC == "0.000"] <- "-"
# seThetaAIC <- 2*(1-pnorm(abs(summary(oIQRaic)$coef/summary(oIQRaic)$se)))
# seThetaAIC <- ifelse(round(seThetaAIC, 3) < .05, "*", "")
# seThetaAIC[is.na(seThetaAIC)] <- ""
# tabAIC <- sapply(seq_len(6), function(i){
#   paste0(tabAIC[,i], "(", tabAIC[,i+6], ")", seThetaAIC[,i])
# })
# pvalX <- format(round(summary(oIQRaic)$test.x[,3],3), nsmall = 3, trim = T)
# pvalX[pvalX == "NA"] <- ""
# tabAIC <- cbind(tabAIC, pvalX)
# colnames(tabAIC) <- c("1", "log(p)", "log(1-p)", "SLP(p,3)[1]", "SLP(p,3)[2]", 
#                       "SLP(p,3)[3]", "P-value")


tabBIC <- format(round(cbind(summary(oIQRbic)$coef, summary(oIQRbic)$se), 3), nsmall = 3, trim = T)
tabBIC[tabBIC == "0.000"] <- "-"
seThetaBIC <- 2*(1-pnorm(abs(summary(oIQRbic)$coef/summary(oIQRbic)$se)))
seThetaBIC <- ifelse(round(seThetaBIC, 3) < .05, "*", "")
seThetaBIC[is.na(seThetaBIC)] <- ""
tabBIC <- sapply(seq_len(6), function(i){
  paste0(tabBIC[,i], "(", tabBIC[,i+6], ")", seThetaBIC[,i])
})
pvalX <- format(round(summary(oIQRbic)$test.x[,3],3), nsmall = 3, trim = T)
pvalX[pvalX == "NA"] <- ""
tabBIC <- cbind(tabBIC, pvalX)
colnames(tabBIC) <- c("1", "log(p)", "log(1-p)", "SLP(p,3)[1]", "SLP(p,3)[2]", 
                      "SLP(p,3)[3]", "P-value")

#### figure 5 ####
ylab <- sapply(1:10, function(j) bquote(beta[.(j-1)](p)))
labels <- c("Intercept", "Age", "Height", "BMI", "Sex", "Smoker", "Exposure", "Cough", "Wheezing", "Asthma")
rownames(oIQRbic$coefficients) <- labels
color <- gray.colors(4)

ind <- which(oIQRbic$s[,1] == 1)
oRQ <- rq(ic ~ ., data=X[, c(ind, 10)], tau=seq(.01,.99,.01))

# pdf("~/Desktop/modBIC_new.pdf", width=.5*20, height=.5*15)
par(mfrow=n2mfrow(length(ind))[c(2,1)], mar=c(5,6,4,1)+.1)
sapply(1:length(ind), function(i){
  plot(oIQRbic, which=ind[i], lwd=2.5, cex.lab=1.5, cex.axis=1.3, xlab="", ylab="",
       ylim=range(c(predict(oIQRbic)[[ind[i]]]$low, coef(oRQ)[i, ], predict(oIQRbic)[[ind[i]]]$up)), col=color[1])
  mtext(bquote(hat(beta)(p)), side = 2, line = 2.5, cex = 1.3)
  mtext("p", side = 1, line = 2.5, cex = 1.3)
  lines(seq(.01,.99,.01), coef(oRQ)[i, ], col=color[2], lwd=2, lty=1)
  abline(h=0, lty=3, col=adjustcolor("black", alpha.f = 0.6))})
# dev.off()
