#### Simulation scenarios 1-2 and 1*-2* 

#### main functions and packages ####

library(qrcmNP)
library(rqPen)
library(Matrix)
library(truncnorm)

modelX <- function(n, p, rho=.5, scale=TRUE){
  Sigma <- forceSymmetric(t(rho^outer(1:p,1:p,"-")))
  cSigma <- chol(Sigma)
  X <- as.matrix(replicate(p, rtruncnorm(n, 0, 1)) %*% cSigma)
  if(scale) X <- scale(X)
  return(X)
}

index <- function(tab){
  recall <- sens <- TPR <- tab[4]/(tab[4]+tab[3])
  spec <- tab[1]/(tab[1]+tab[2])
  FPR <- 1-spec
  precision <- tab[4]/(tab[4]+tab[2])
  acc <- sum(diag(tab))/sum(tab)
  return(list(TPR=TPR, FPR=FPR, recall=recall, precision=precision, acc=acc, sens=sens, spec=spec))
}

F1 <- function(y,X,theta,B,Q){
  n <- nrow(X)
  p <- rep(0.5,n)
  for(i in 2:20){
    p <- c(p + 0.5^i*sign(y - Q(p,theta,B,X)))
  }
  p
}

F2 <- function(y,X,beta){
  n <- nrow(X)
  p <- rep(0.5,n)
  for(i in 2:20){
    p <- c(p + 0.5^i*sign(y - X %*% beta))
  }
  p
}

roc3 <- function(IND, INDRQPEN, nl=100, ns=2, np=5){
  lab <- c(expression(QRCM[p](sim~1)), 
           expression(QRCM[p](sim~1^"*")))
  for(i in 1:length(pBias)) lab <- c(lab, bquote(QR[p]~","~p==.(pBias[i])))
  # lab <- c("PQRCM (sim 1)", "PQRCM (sim 2)", paste0("PQR, p = ", pBias))
  col <- grey.colors(ns+np)
  # par(mfrow=c(1,1))
  o <- sapply(1:nl, function(.i) colMeans(IND[,.i,1,1:2], na.rm=T))
  plot(o[2,], o[1,], lwd=3, type="l", cex.axis=1.5, cex.lab=1.5, col=col[1], xlab="FPR = 1 - Specificity", ylab="TPR = Sensitivity")
  for(i in 2:ns){
    o <- sapply(1:nl, function(.i) colMeans(IND[,.i,i,1:2], na.rm=T))
    lines(o[2,], o[1,], lwd=3, lty=i, col=col[i])
  }
  for(i in 1:np){
    o <- sapply(1:nl, function(.i) colMeans(INDRQPEN[,.i,i,1:2], na.rm=T))
    lines(o[2,], o[1,], lwd=3, lty=(ns+i), col=col[(ns+i)])
  }
  legend("bottomright", legend=lab, lty=1:(ns+np), col=col, bty="n", lwd=3, cex=1.5)
}
roc3_2 <- function(IND, INDRQPEN, nl=100, ns=2, np=5){
  lab <- c(expression(QRCM[p](sim~2)), 
           expression(QRCM[p](sim~2^"*")))
  for(i in 1:length(pBias)) lab <- c(lab, bquote(QR[p]~","~p==.(pBias[i])))
  # lab <- c("PQRCM (sim 1)", "PQRCM (sim 2)", paste0("PQR, p = ", pBias))
  col <- grey.colors(ns+np)
  # par(mfrow=c(1,1))
  o <- sapply(1:nl, function(.i) colMeans(IND[,.i,1,1:2], na.rm=T))
  plot(o[2,], o[1,], lwd=3, type="l", cex.axis=1.5, cex.lab=1.5, col=col[1], xlab="FPR = 1 - Specificity", ylab="TPR = Sensitivity")
  for(i in 2:ns){
    o <- sapply(1:nl, function(.i) colMeans(IND[,.i,i,1:2], na.rm=T))
    lines(o[2,], o[1,], lwd=3, lty=i, col=col[i])
  }
  for(i in 1:np){
    o <- sapply(1:nl, function(.i) colMeans(INDRQPEN[,.i,i,1:2], na.rm=T))
    lines(o[2,], o[1,], lwd=3, lty=(ns+i), col=col[(ns+i)])
  }
  legend("bottomright", legend=lab, lty=1:(ns+np), col=col, bty="n", lwd=3, cex=1.5)
}

#### initialization step ####
set.seed(1234)
n <- 500
q <- 15
k <- 4

### generating model matrix without correlation structure
### used for simulations 1 and 2
X <- modelX(n, q, 0, FALSE)

### uncomment this line to generate model matrix with toeplitz correlation structure
### used for simulations 1* and 2*
# X <- modelX(n, q, .5, FALSE)
rownames(X) <- 1:n
colnames(X) <- paste("X", 1:q, sep="")

### verifing regularity conditions
sqrt(2*log(q)) # 2.33
integrate(function(x){1 + 1*qnorm(x)}, 0, 1)
integrate(function(x){-1 + 2*log(x)}, 0, 1)
integrate(function(x){2 + 2*x}, 0, 1)
integrate(function(x){2 + 2*sqrt(x)}, 0, 1)
integrate(function(x){3 + 1*qnorm(x)}, 0, 1)
theta <- matrix(c(1, 1, 0, 0, 0,
                  -1, 0, 0, 2, 0, #X1
                  2, 0, 2, 0, 0,  #X2
                  2, 0, 0, 0, 2,  #X3
                  3, 1, 0, 0, 0,  #X4
                  rep(0, 5*11)),
                ncol=k+1, byrow=T)

rownames(theta) <- c("(intercept)", paste("X", 1:q, sep=""))
colnames(theta) <- c("(intercept)", "qnorm(p)", "p", "log(p)", "sqrt(p)")

### defining useful functions
B <- function(p){matrix(cbind(1, qnorm(p), p, log(p), sqrt(p)), nrow=5, byrow=T)}
B2 <- function(p){matrix(cbind(1, qnorm(p), slp(p,3)), nrow=5, byrow=T)}
Q <- function(p, theta, B, X){rowSums(X * t(theta %*% B(p)))}
Qbias <- function(p, theta, B, X){as.matrix(X) %*% (theta %*% B(p))}

### true values of quantile function 
pBias <- c(.01, .1, .5, .90, .99)
yTrue <- Qbias(pBias, theta, B, cbind(1, X))
thetaTrue <- factor(1*apply(theta, 1, function(.x) sum(.x != 0) > 0), levels=c("0","1"))

### defining maximal model for simulation scenario 1 and 1*
s <- matrix(1, q+1, k+1); s[1, 3:(k+1)] <- 0; s[c(2:4,6:(q+1)), 2] <- 0; s[c(2,4:(q+1)), 3] <- 0; s[3:(q+1), 4] <- 0;
s[c(2:3,5:(q+1)), 5] <- 0; s[6:8, 2] <- 1; s[9:11, 3] <- 1; s[12:14, 4] <- 1; s[15:16, 5] <- 1
rownames(s) <- c("(intercept)", paste("X", 1:q, sep=""))
colnames(s) <- c("(intercept)", "qnorm(p)", "p", "log(p)", "sqrt(p)")

### defining maximal model for simulation scenario 2 and 2*
s2 <- matrix(1, q+1, k+1); s2[1, 3:(k+1)] <- 0; s2[2:(q+1), 2] <- 0
rownames(s2) <- c("(intercept)", paste("X", 1:q, sep=""))
colnames(s2) <- c("(intercept)", "qnorm(p)", "slp(p,1)", "slp(p,2)", "slp(p,3)")

M <- 1000
set.seed(1234)

### 100 lambda values to determine roc curves
lambda.seq <- seq(170, .01, l=100)
lambda.seq2 <- seq(30, .0001, l=100)

### allocating output objects
IND <- array(NA, dim=c(M, length(lambda.seq), 2, 9), dimnames=list(1:M, round(lambda.seq,2), c("s","s2"), c("TPR","FPR","REC","PREC","ACC","SENS","SPEC","DFaic","DFbic")))
INDRQPEN <- array(NA, dim=c(M, length(lambda.seq2), length(pBias), 9), dimnames=list(1:M, round(lambda.seq2,2), pBias, c("TPR","FPR","REC","PREC","ACC","SENS","SPEC","DFaic","DFbic")))
P <- array(NA, dim=c(M, 6, n), dimnames=list(1:M, c("TRUE","AIC1","BIC2","AIC1","BIC2","IQR"), 1:n))
Y <- array(NA, dim=c(M, n, 7, length(pBias)), dimnames=list(1:M, 1:n, c("AIC1","BIC1","AIC2","BIC2","IQR","RQ1","RQ2"), pBias))
MSE <- array(NA, dim=c(M, 7, length(pBias)), dimnames=list(1:M, c("AIC1","BIC1","AIC2","BIC2","IQR","RQ1","RQ2"), pBias))

betaRQ <- matrix(NA, q+1, length(lambda.seq2))
aicRQ <- bicRQ <- vector(l=length(lambda.seq2))

#### body of the simulation ####
pb <- txtProgressBar(min=0, max=M, style=3)
for(j in 1:M){
  setTxtProgressBar(pb, j)
  
  ### generating response
  P[j,1,] <- pp <- runif(n)
  y <- Q(p=pp, theta=theta, B=B, X=cbind(1, X))
  
  ### computing penalized qrcm models according with b(p) basis of simulations 1 and 2
  obj <- piqr(y~ ., data=as.data.frame(X), formula.p = ~ qnorm(p) + p + I(log(p)) + I(sqrt(p)), s=s, display=FALSE, lambda=lambda.seq)
  obj2 <- piqr(y ~ ., data=as.data.frame(X), formula.p = ~ qnorm(p) + slp(p,3), s=s2, display=FALSE, lambda=lambda.seq)
  
  ### some plots only for the first iteration
  if(j == 1){
    par(mfrow=c(2,2));
    plot(obj, xvar="grad"); plot(obj, xvar="lambda")
    plot(obj2, xvar="grad"); plot(obj2, xvar="lambda")
    par(mfrow=c(1,1));
  }
  
  plott <- if(j == 1) TRUE else FALSE
  par(mfrow=c(2,2));
  bestObj1 <- gof.piqr(obj, method="AIC", plot=plott); bestObj2 <- gof.piqr(obj, method="BIC", plot=plott)
  bestObj3 <- gof.piqr(obj2, method="AIC", plot=plott); bestObj4 <- gof.piqr(obj2, method="BIC", plot=plott)
  par(mfrow=c(1,1));
  
  ### true model, unpenalized qrcm model
  obj3 <- iqr(y ~ ., data=as.data.frame(X), formula.p = ~ qnorm(p) + p + I(log(p)) + I(sqrt(p)), s=1*I(theta != 0))
  
  ### allocating quantile functions, one for each model
  Y[j, , 1, ] <- Qbias(pBias, bestObj1$betaMin, B, cbind(1, X))
  Y[j, , 2, ] <- Qbias(pBias, bestObj2$betaMin, B, cbind(1, X))
  Y[j, , 3, ] <- Qbias(pBias, bestObj3$betaMin, B2, cbind(1, X))
  Y[j, , 4, ] <- Qbias(pBias, bestObj4$betaMin, B2, cbind(1, X))
  Y[j, , 5, ] <- Qbias(pBias, obj3$coefficients, B, cbind(1, X))
  
  ### allocating number of selected covariates by the penalized qrcm models
  dfAic <- sum(I(bestObj1$betaMin[,1] != 0))
  dfBic <- sum(I(bestObj2$betaMin[,1] != 0))
  dfAic2 <- sum(I(bestObj3$betaMin[,1] != 0))
  dfBic2 <- sum(I(bestObj4$betaMin[,1] != 0))
  
  ### computing Sensitivity and Specificity to build the roc curves for the penalized qrcm models
  temp <- lapply(obj$coefficients, function(.x) factor(1*I(.x[,1] != 0), levels=c("0","1")))
  temp2 <- lapply(obj2$coefficients, function(.x) factor(1*I(.x[,1] != 0), levels=c("0","1")))
  for(.ii in 1:length(lambda.seq)){
    tab <- table(pred=temp[[.ii]], true=thetaTrue)
    tab2 <- table(pred=temp2[[.ii]], true=thetaTrue)
    IND[j, .ii, 1, ] <- c(unlist(index(tab)), dfAic, dfBic)
    IND[j, .ii, 2, ] <- c(unlist(index(tab2)), dfAic2, dfBic2)
  }
  
  ### computing quantile functions, number of selected covariates and Sensitivity and Specificity
  ### for standard penalized quantile regression
  ### moreover, we compute measures of MSE of the CDF for all models
  for(.i in 1:length(pBias)){
    yy <- Qbias(pBias[.i], theta, B, cbind(1,X))
    
    for(.ii in 1:length(lambda.seq2)){
      obj4 <- rq(y~X, method="lasso", tau=pBias[.i], lambda=lambda.seq2[.ii])
      betaRQ[, .ii] <- ifelse(abs(obj4$coefficients) >= 1e-6, obj4$coefficients, 0)
      temp <- factor(1*I(betaRQ[, .ii] != 0), levels=c("0","1"))
      tab <- table(pred=temp, true=thetaTrue)
      INDRQPEN[j,.ii,.i,1:7] <- unlist(index(tab))
      aicRQ[.ii] <- log(obj4$rho) + sum(betaRQ[, .ii] != 0)/n
      bicRQ[.ii] <- log(obj4$rho) + log(n)*sum(betaRQ[, .ii] != 0)/(2*n)
    }
    ind <- which.min(aicRQ)
    ind2 <- which.min(bicRQ)
    
    betaRQ1 <- betaRQ[, ind]
    betaRQ2 <- betaRQ[, ind2]
    
    Y[j, , 6, .i] <- cbind(1,X) %*% betaRQ1
    Y[j, , 7, .i] <- cbind(1,X) %*% betaRQ2
    
    MSE[j, 1, .i] <- mean((F1(yy, cbind(1,X), bestObj1$betaMin, B, Q) - pBias[.i])^2)
    MSE[j, 2, .i] <- mean((F1(yy, cbind(1,X), bestObj2$betaMin, B, Q) - pBias[.i])^2)
    MSE[j, 3, .i] <- mean((F1(yy, cbind(1,X), bestObj3$betaMin, B2, Q) - pBias[.i])^2)
    MSE[j, 4, .i] <- mean((F1(yy, cbind(1,X), bestObj4$betaMin, B2, Q) - pBias[.i])^2)
    MSE[j, 5, .i] <- mean((F1(yy, cbind(1,X), obj3$coefficients, B, Q) - pBias[.i])^2)
    MSE[j, 6, .i] <- mean((F2(yy, cbind(1,X), betaRQ1) - pBias[.i])^2)
    MSE[j, 7, .i] <- mean((F2(yy, cbind(1,X), betaRQ2) - pBias[.i])^2)
    
    INDRQPEN[j, , .i, 8] <- sum(betaRQ1 != 0)
    INDRQPEN[j, , .i, 9] <- sum(betaRQ2 != 0)
  }
  
  ### saving CDF for the penalized qrcm models and for the unpenalized qrcm model
  P[j, 2, ] <- predict(obj, bestObj1$minLambda, type="CDF")[, 1]
  P[j, 3, ] <- predict(obj, bestObj2$minLambda, type="CDF")[, 1]
  P[j, 4, ] <- predict(obj2, bestObj3$minLambda, type="CDF")[, 1]
  P[j, 5, ] <- predict(obj2, bestObj4$minLambda, type="CDF")[, 1]
  P[j, 6, ] <- predict(obj3, type="CDF")[, 1]
}
close(pb)

#### figure 2 ####
### roc3 is used for the scenario without correlation structure
### roc3_2 is used for the scenario with the toeplitz matrix as correlation structure
pdf("~/Desktop/roc_piqr.pdf", width=.6*12, height=.6*12)
par(mfrow=c(1,1))
roc3(IND, INDRQPEN, nl=100, ns=2, np=5)
# roc3_2(IND, INDRQPEN, nl=100, ns=2, np=5)
dev.off()

#### figure 3 ####
pdf("~/Desktop/boxplot_df_piqr.pdf", width=.8*18, height=.8*10)
par(mfrow=c(1,1))
boxplot(cbind(IND[,1,1,8:9],IND[,1,2,8:9],INDRQPEN[,1,1,8:9],INDRQPEN[,1,2,8:9],INDRQPEN[,1,3,8:9],
              INDRQPEN[,1,4,8:9],INDRQPEN[,1,5,8:9]), names=rep(c("AIC", "BIC"),7),
        at=c(1,2,4,5,7,8,10,11,13,14,16,17,19,20), cex.axis=1.2, col=grey.colors(2))
abline(h=5, lty=2, col="grey")
mtext(expression(QRCM[p](sim~1)), side=1, at=1.5, line=2.5, cex=1.2)
mtext(expression(QRCM[p](sim~1^"*")), side=1, at=4.5, line=2.5, cex=1.2)
mtext(bquote(QR[p]~","~p==.(pBias[1])), side=1, at=7.5, line=2.5, cex=1.2)
mtext(bquote(QR[p]~","~p==.(pBias[2])), side=1, at=10.5, line=2.5, cex=1.2)
mtext(bquote(QR[p]~","~p==.(pBias[3])), side=1, at=13.5, line=2.5, cex=1.2)
mtext(bquote(QR[p]~","~p==.(pBias[4])), side=1, at=16.5, line=2.5, cex=1.2)
mtext(bquote(QR[p]~","~p==.(pBias[5])), side=1, at=19.5, line=2.5, cex=1.2)
dev.off()

#### table 2 #####
MSE2 <- apply(MSE, 3, colMeans, na.rm=T)[c(5,1:4,6:7),]
SE2 <- apply(MSE, 3, function(.x) apply(.x, 2, sd, na.rm=T))[c(5,1:4,6:7),]
tabMSE <- format(round(cbind(MSE2, SE2), 3), nsmall = 3, trim = T)
tabMSE <- sapply(seq_len(5), function(i){
  paste0(tabMSE[,i], "(", tabMSE[,i+5], ")")
})
rownames(tabMSE) <- c("QRCM", "QRCMp-AIC (sim 1)", "QRCMp-BIC (sim 1)",
                      "QRCMp-AIC (sim 1*)", "QRCMp-BIC (sim 1*)",
                      "QRp-AIC", "QRp-BIC")
colnames(tabMSE) <- colnames(MSE2)
